% Crackle_Alignment_Gradient_Ascent.m adjusts the 4 angular degrees of 
% freedom of mirrors so that fringe contrast approaches as closely to unity 
% as possible. Empirically the algorithm starts to oscillate when any of 
% the perturbation is > 20e-4 rad (when noise is not considered). 
% 
% if 1% noise is included, graph oscillates at low power because of small 
% SNR. If any of the misalignment >7e-4, there is high probability that
% code will oscillate indefinitely.
% 
% The key to the algorithm below is choosing suitably sized steps to avoid
% 'crossing over' to the other half of the predicted Gaussian pattern
% (fringe contrast increases and then decreases because it crossed over the
% other half of the graph)

%% Compile and create object
clear classes
MIST('Crackle_Michelson.mist');
m = Michelson(6);

%% Set initial conditions
% Define global variables
global power_perror gainap gainsp N phi

% perturbation constants (x1 means misalignment along x axis of mirror 1)
perturbx1 = 7e-4;
perturby1 = 7e-4;
perturbx2 = 6e-4;
perturby2 = 5e-4;

power_perror = 0; % percentage power noise error
gainap = 1; % AP gain
gainsp = 1; % SP gain
N = 40; % number of points for fringe pattern
phi = linspace(0, pi, N); % generate linear points for phase propagation
step = 1e-5; % unit step of angular displacement
i=1; % initiate number of iterations

%% First measurement of fringe contrast
m.m1.tx = perturbx1; % set perturbation on x1
m.m1.ty = perturby1; % set perturbation on y1
m.m2.tx = perturbx2; % set perturbation on x2
m.m2.ty = perturby2; % set perturbation on y2
fc = f_contrast(m); % measure initial fringe contrast
    
%% Iteration
x1_initial = m.m1.tx; % memory for x1 initial condition
y1_initial = m.m1.ty; % memory for y1 initial condition
x2_initial = m.m2.tx; % memory for x2 initial condition
y2_initial = m.m2.ty; % memory for y2 initial condition

alignment(1,i) = x1_initial; % x1 misalignment
alignment(2,i) = y1_initial; % y1 misalignment
alignment(3,i) = x2_initial; % x2 misalignment
alignment(4,i) = y2_initial; % y2 misalignment
alignment(5,i) = i; % loop number
alignment(6,i) = fc % fringe contrast after moving dof

tic
while fc<0.998 % threshold (arbitrary)
i=i+1; %increase step by 1
%% calculate X1 local gradient
m.m1.tx = x1_initial + step; % move in +ve direction
fcx1= f_contrast(m); % compute fringe contrast

if fcx1 < fc %if fringe contrast decreases
    m.m1.tx = x1_initial - step; %reverse direction
    fcx1 = f_contrast(m); % compute fringe contrast
end

gx1 = (fcx1-fc)/(m.m1.tx-x1_initial); % compute gradient
m.m1.tx = x1_initial; % return to original position, so it doesnt affect
% subsequent measurements of gradients
    
%% calculate Y1 local gradient
m.m1.ty = y1_initial + step; % move in +ve direction
fcy1= f_contrast(m); % compute fringe contrast

if fcy1 < fc %if fringe contrast decreases
    m.m1.ty = y1_initial - step; %reverse direction
    fcy1 = f_contrast(m); % compute fringe contrast
end

gy1 = (fcy1-fc)/(m.m1.ty-y1_initial); % compute gradient
m.m1.ty = y1_initial; % return to original position, so it doesnt affect
% subsequent measurements of gradients
    
%% calculate X2 local gradient
m.m2.tx = x2_initial + step; % move in +ve direction
fcx2= f_contrast(m); % compute fringe contrast

if fcx2 < fc %if fringe contrast decreases
    m.m2.tx = x2_initial - step; %reverse direction
    fcx2= f_contrast(m); % compute fringe contrast
end

gx2 = (fcx2-fc)/(m.m2.tx-x2_initial); % compute gradient
m.m2.tx = x2_initial; % return to original position, so it doesnt affect
% subsequent measurements of gradients
    
%% calculate Y2 local gradient
m.m2.ty = y2_initial + step; % move in +ve direction
fcy2= f_contrast(m); % compute fringe contrast

if fcy2 < fc %if fringe contrast decreases
    m.m2.ty = y2_initial - step; %reverse direction
    fcy2= f_contrast(m); % compute fringe contrast
end

gy2 = (fcy2-fc)/(m.m2.ty-y2_initial); % compute gradient
m.m2.ty = y2_initial; % return to original position

%% Determine dof with steepest gradient
 
gain = [gx1, gy1, gx2, gy2]; % vector of all gains

%% Adjust step size multiple 'num', depending on fringe contrast measured
% crucial to balance between speed and possibility of stepping over the
% other end of the graph

if fc <= 0.5
    num=33*gain/1075;
elseif fc> 0.5 && fc <= 0.90 
    num=12*gain/1075;
elseif fc>0.90 && fc <= 0.95
    num=8*gain/1075;
elseif fc>0.95
    num=2*gain/1075;
end

%% Move mirror along dof with steepest gradient

a = [x1_initial, y1_initial, x2_initial, y2_initial];
a = a + num*step;

m.m1.tx = a(1);
x1_initial = m.m1.tx; % change initial condition in next loop
m.m1.ty = a(2);
y1_initial = m.m1.ty; % change initial condition in next loop
m.m2.tx = a(3);
x2_initial = m.m2.tx; % change initial condition in next loop
m.m2.ty = a(4);
y2_initial = m.m2.ty; % change initial condition in next loop

fc = f_contrast(m); % reset fringe contrast for next loop

%% Store Data

alignment(1,i) = x1_initial; % x1 misalignment
alignment(2,i) = y1_initial; % y1 misalignment
alignment(3,i) = x2_initial; % x2 misalignment
alignment(4,i) = y2_initial; % y2 misalignment
alignment(5,i) = i; % loop number
alignment(6,i) = fc; % fringe contrast after moving dof

toc
end

% Plot graph of fringe contrast against number of steps
figure()
plot(alignment(5,:), alignment(6,:), 'b--o')
xlabel('Steps', 'FontSize', 18);
ylabel('Fringe Contrast', 'FontSize', 18);

figure()
plot(alignment(5,:), alignment(1,:)*10^3, 'g-o')
hold all
plot(alignment(5,:), alignment(2,:)*10^3, 'r-o')
hold all
plot(alignment(5,:), alignment(3,:)*10^3, 'b-x')
hold all
plot(alignment(5,:), alignment(4,:)*10^3, 'k-x')
hold all
xlabel('Steps', 'FontSize', 18);
ylabel('Angular displacement [mrad]', 'FontSize', 18);
legend('X1', 'Y1', 'X2', 'Y2');
grid on
