% Crackle_Alignment_Gradual_Step.m assumes the 4 angular degrees of
% freedom of mirror misalignment is independent of one another. It scans
% through the fringe contrast pattern at fixed intervals, measure the
% fringe contrast, locate the point of highest fringe contrast, and taking
% fixed steps to maximize fringe contrast. Superceded by gradient ascent
% method.

%% Compile and create object
clear classes
MIST('Crackle_Michelson.mist');
m = Michelson(6);

%% Set initial conditions
% Define global variables
global power_perror gainap gainsp N phi

% perturbation constants (x1 means misalignment along x axis of mirror 1)
perturbx1 = 7e-4;
perturby1 = -3e-4;
perturbx2 = -6.4e-4;
perturby2 = 5e-4;

power_perror = 0.01; % percentage power noise error
gainap = 1; % AP gain
gainsp = 1; % SP gain
N = 40; % number of points for fringe pattern
phi = linspace(0, pi, N); % generate linear points for phase propagation
step = 1e-5; % unit step of angular displacement

% approximately 5 sigma range
lower = -0.003; % scanning lower limit
upper = 0.003; % scanning upper limit

m.m1.tx = perturbx1; % set perturbation on x1
m.m1.ty = perturby1; % set perturbation on y1
m.m2.tx = perturbx2; % set perturbation on x2
m.m2.ty = perturby2; % set perturbation on y2


%% Locate the point to somewhere on the peak along x1
% strategy: quickly sample across range to start half way

nptquickx1 = 10;
% Scanning through spectrum around the perturbed misalignment
quicksamplex1 = linspace(perturbx1+lower,perturbx1+upper, nptquickx1); 
quickanswerx1 = zeros(size(nptquickx1),2); % matrix to store data

% Determine fringe contrast pattern
for p=1:numel(quicksamplex1)
    m.m1.tx = quicksamplex1(p);
    quickanswerx1(p,1)= m.m1.tx; % change x1 angular alignment
    quickanswerx1(p,2)= f_contrast(m); % fringe contrast
end

figure()
plot(quickanswerx1(:,1), quickanswerx1(:,2), 'x-')
xlabel('Angular displacement [mrad]', 'FontSize', 18)
ylabel('Fringe contrast', 'FontSize', 18)

fcx1 = max(quickanswerx1(:,2)); % Pick point with max fcontrast
indexx1 = find(quickanswerx1(:,2)==fcx1); % Determine index of point
m.m1.tx = quickanswerx1(indexx1, 1); % Move to the point with maximum power

%% Maximising power along x1

fcx1_store = 0; % as a temporary memory, set initial value to be zero
i=0; % initiate number of iterations

% Plot Fringe contrast against Step
figure(100) % arbitrarily high value to avoid overlap with other smaller value graphs
xlabel('Step', 'FontSize', 18)
ylabel('Fringe contrast', 'FontSize', 18)
hold all
% Plot Angular error signal against Step
figure(101)
xlabel('Step', 'FontSize', 18)
ylabel('Angular error signal', 'FontSize', 18)
hold all
tic

while abs(fcx1 - fcx1_store)>0.001 % threshold (arbitrary)

    fcx1_store = fcx1;
    i=i+1;
    m.m1.tx = m.m1.tx + step; % shift in + direction
    clear APx1_power SPx1_power;
    trialx1 = f_contrast(m); % meaasure fringe contrast

    if trialx1<=fcx1 % if power decreases or is equal
        m.m1.tx = m.m1.tx - 2*step; % shift in opposite direction
        clear APx1_power SPx1_power;
        trialx1 = f_contrast(m); % measure fringe contrast
    end;

    fcx1 = trialx1;
    fringex1(1,i) = fcx1;
    fringex1(2,i) = m.m1.tx;

figure(100) % arbitrarily high value to avoid overlap with other smaller value graphs
plot(i, fringex1(1,i), 'b--o')
hold all
pause(0.1)

figure(101)
plot(i, fringex1(2,i), 'r--o')
hold all
pause(0.1)

end
toc

%% Locate the point to somewhere on the peak along y1
% strategy: quickly sample across range to start half way

nptquicky1 = 10;
quicksampley1 = linspace(perturby1 + lower, perturby1 + upper, nptquicky1); % Scanning through spectrum around the perturbed misalignment
quickanswery1 = zeros(size(nptquicky1),2);
tic

% Determine fringe contrast pattern
for p=1:numel(quicksampley1)
    m.m1.ty = quicksampley1(p);
    quickanswery1(p,1)= m.m1.ty;
    quickanswery1(p,2)= f_contrast(m); % fringe contrast;
end
toc

figure()
plot(quickanswery1(:,1), quickanswery1(:,2))
initialy1 = max(quickanswery1(:,2)); % Must have sampled at somewhere at near the peak
indexy1 = find(quickanswery1(:,2)==initialy1);
m.m1.ty = quickanswery1(indexy1, 1); % Move alignment to the point with maximum power
xlabel('Angular displacement [rad]')
ylabel('Fringe contrast')

%% Maximising power along y1

step = 1e-5; % unit step of angular control in x1
fcy1 = initialy1; % measured fringe contrast
fcy1_store = 0; % as a temporary memory, set initial value to be zero
i=0; % initial step value

% Plot Fringe contrast against Step
figure(200) % arbitrarily high value to avoid overlap with other smaller value graphs
xlabel('Step')
ylabel('Fringe contrast')
hold all

% Plot Angular error signal against Step
figure(201)
xlabel('Step')
ylabel('Angular error signal')
hold all

tic
while abs(fcy1 - fcy1_store)>0.001 % threshold, artificial

    fcy1_store = fcy1;
    i=i+1;
    m.m1.ty = m.m1.ty + step; % shift in + direction
    clear APy1_power SPy1_power;
    trialy1 = f_contrast(m);
    
    if trialy1<=fcy1 % if power decreases or is equal
        m.m1.ty = m.m1.ty - 2*step; % shift in opposite direction
        clear APy1_power SPy1_power;
        trialy1 = f_contrast(m);
    end;

    fcy1 = trialy1;
    fringey1(1,i) = fcy1;
    fringey1(2,i) = m.m1.ty;

figure(200) % arbitrarily high value to avoid overlap with other smaller value graphs
plot(i, fringey1(1,i), 'b--o')
hold all
pause(0.1)

figure(201)
plot(i, fringey1(2,i), 'r--o')
hold all
pause(0.1)
end
toc