% Crackle_Frequency_Noise_Minimization.m
% Same as Crackle_Frequency_Noise.m EXCEPT in simulation m.s3.L doesn't 
% change. This is to reflect on the fact that in the real experiment, only 
% one arm is moved. Minimization algorithm is also included below.

%% Compile and create object
clear classes
MIST('Crackle_Michelson.mist');
m = Michelson(6);

%% compute Michelson tranfer function of longitudinal motion of mirrors

% Modulation depth of differential motion of mirrors
m.z_d2_m1 = 1; % mirror 1
m.z_d2_m2 = -1; % mirror 2

fr = logspace(0,3,100); % Generate a set of logarithmic points
T = m.TransferFunction('d2', {'sy','asy'}, fr); % Transfer function

% calculate average (SP-AP) transfer function
% real to eliminate imaginary part that arises due to computation error 
tfmich = real(mean(T(1,:)-T(2,:))); 

% plot transfer function of AP and SP
figure()
subplot(211)
semilogx(fr, real(T(1,:)), fr, real(T(2,:)));
legend('SP', 'AP')
ylabel('Abs [W/m]', 'FontSize', 18)
grid
hold all
subplot(212)
semilogx(fr, angle(T(1,:)), fr, angle(T(2,:)));
legend('SP', 'AP');
grid
ylabel('Phase [rad]', 'FontSize', 18)
xlabel('Frequency [Hz]', 'FontSize', 18)

%% %% Laser frequency gain against macroscopic length difference
% Graph computed is superposed with simulation after noise is added to
% compare

m.las.setMotionShape(0.5i); % imaginary value sets frequency modulation
m.z_d1_las = 1; % arbitrary modulation depth
m.f_d1 = 1000; % arbitrary modulation frequency

% Generate a set of points
pt = 100;
dL = linspace(0,0.02, pt); % dL is half the macroscopic length difference
data = zeros(pt, 3); % just a matrix to store my data
tf1 = zeros(size(pt)); % transfer function calculated at asymmetric port
tf2 = zeros(size(pt)); % transfer function calculated at symmetric port

for k=1:pt
    % set the length (only change length of movable arm)
    m.s2.L = 0.316 + dL(k);
    m.s3.L = 0.316;
    % compute the optical TF
    tf1(k) = m.asy_TF_d1()/ (1j*m.f_d1); % divide TF by modulation freq
    tf2(k) = m.sy_TF_d1() / (1j*m.f_d1); % divide TF by modulation freq
    data(k,1) = m.s2.L;
    data(k,2) = abs(tf1(k) - tf2(k))./tfmich; 
    % gaind2 is calculated above. This is to calibrate TF wrt mirror
    % displacement
end

%% Minimization Algorithm

for a=1:100

l_error = 10e-6; % absolute error of macroscopic length 
tf_error = 0.05; % percentage error of transfer function
tf_tolerance = 0.1e-17; % threshold for laser frequency gain (arbitrary)
length_step = 1e-3; % step of length shift (should be larger than the 
% random error of length), if too small, measurement may be dominated by 
% noise.
tfdiff_1= 1; % set initial value for first sample point
tfdiff_2= 1; % set initial value for second sample point
tfdiff_3= 1; % set initial value for estimated point
i=0; % initiate number of iterations

% Perturbation/initial condition
m.s2.L = uniform_noise(l_error,0.328);%unequal macroscopic length (arbitrary)
m.s3.L = 0.316; % fixed arm

% real time plot generated by while loop
figure(300)
xlabel('Step', 'FontSize', 18)
ylabel('Macroscopic length of moving arm [m]', 'FontSize', 18)
grid on

clear fnoise

while abs(tfdiff_1) > tf_tolerance || abs(tfdiff_2) > tf_tolerance || ...
      abs(tfdiff_3) > tf_tolerance 
    
    % First sample point
    i=i+1;
    distance1 = m.s2.L; % memory
    tf1_1 = percentage_noise(tf_error,m.asy_TF_d1()/ (1j*m.f_d1)); % TF
    tf2_1 = percentage_noise(tf_error,m.sy_TF_d1() / (1j*m.f_d1)); % TF
    % TF difference, real to eliminate imaginary part
    tfdiff_1 = real(tf2_1-tf1_1)./tfmich; 
    fnoise(i,1) = distance1; % store data
    fnoise(i,2) = tfdiff_1; % store data
    plot(i, fnoise(i,1), 'r--o') % plot first point with red
    hold all
    pause(0.1)
    if abs(tfdiff_1) < tf_tolerance % stop if threshold is met 
        break % break from loop
    end
    
    % Second sample point
    i=i+1;
    % move arm in -ve direction
    m.s2.L = uniform_noise(l_error,distance1 - length_step); 
    distance2 = m.s2.L; % memory
    tf1_2 = percentage_noise(tf_error,m.asy_TF_d1()/ (1j*m.f_d1)); % TF
    tf2_2 = percentage_noise(tf_error,m.sy_TF_d1() / (1j*m.f_d1)); % TF
    tfdiff_2 = real(tf2_2-tf1_2)./tfmich; % TF difference
    
    % Since in theory, straight line has positive gradient, moving arm in
    % -ve direction should yield second point with tfdiff_2 < tfdiff_1.
    % However noise effect may dominate if length_step is small, and 
    % tfdiff_2 > tfdiff_1, point will be extrapolated to wrong direction
     while tfdiff_2 > tfdiff_1
         clear tf1_2 tf2_2 tfdiff_2
         % keep moving in same direction then resample
         m.s2.L = uniform_noise(l_error,m.s2.L - length_step); 
         distance2 = m.s2.L; % memory
         tf1_2 = percentage_noise(tf_error,m.asy_TF_d1()/ (1j*m.f_d1));%TF
         tf2_2 = percentage_noise(tf_error,m.sy_TF_d1() / (1j*m.f_d1));%TF
         tfdiff_2 = real(tf2_2-tf1_2)./tfmich; % TF difference
     end

    fnoise(i,1) = distance2; % store data
    fnoise(i,2) = tfdiff_2; % store data
    plot(i, fnoise(i,1), 'b--o') % plot second point with blue
    hold all
    pause(0.1)
    if abs(tfdiff_2) < tf_tolerance % stop if threshold is met 
        break % break from loop
    end
    
    % Calculate the equation of straight line between two points
    % gradient, should be positive, because we take SP-AP
    grad = (tfdiff_2 - tfdiff_1)./(distance2 - distance1); 
    c = tfdiff_1 - grad*distance1; % y-intercept
    x_intercept = -c./grad; % x-intercept
    
    % Measure the TF again
    i=i+1;
    m.s2.L = uniform_noise(l_error,x_intercept); % Move arm to x-intercept
    tf1_3 = percentage_noise(tf_error,m.asy_TF_d1()/ (1j*m.f_d1)); % TF
    tf2_3 = percentage_noise(tf_error,m.sy_TF_d1() / (1j*m.f_d1)); % TF
    tfdiff_3 = real(tf2_3-tf1_3)./tfmich; % TF difference
    fnoise(i,1) = m.s2.L; % store data
    fnoise(i,2) = tfdiff_3; % store data
    plot(i, fnoise(i,1), 'g--o') %green to indicate estimated point
    hold all
    pause(0.1)
    if abs(tfdiff_3) < tf_tolerance % stop if threshold is met 
        break % break from loop
    end
    
    % repeat iteration
end

figure(301)
plot(data(:,1), data(:,2)*10^17, 'g')
hold all
plot(fnoise(:,1), fnoise(:,2)*10^17, 'b--o')
hold all
plot(fnoise(1,1), fnoise(1,2)*10^17,'ko' )
hold all
plot(fnoise(i,1), fnoise(i,2)*10^17,'ro')
hold all
legend ('Theoretical', 'Iteration', 'Initial', 'Final')
xlabel('Macroscopic length of movable Arm [m]', 'FontSize', 18)
ylabel('Laser frequency noise gain [10^-^1^7 m/Hz]', 'FontSize', 18)
grid on

test(1,a) = i;
test(2,a) = fnoise(i,1);
test(3,a) = fnoise(i,2);
end

figure()
subplot(311)
hist(test(3,:)*10^18, 50, 'r');
xlabel('Laser frequency noise coupling [10^-^1^8 m/Hz]', 'FontSize', 18)
ylabel('Count', 'FontSize', 18)
hold all
subplot(312)
hist(test(2,:), 50);
xlabel('Macroscopic Length of Movable Arm [m]', 'FontSize', 18)
ylabel('Count', 'FontSize', 18)
hold all
subplot(313)
xvalues=0:18;
hist(test(1,:), xvalues)
xlabel('Number of Steps', 'FontSize', 18)
ylabel('Count', 'FontSize', 18)

