% Crackle_Relative_Gain.m aims to equalize the power readout at both
% ports by adjusting the relative gain between the two ports.

%% Compile and create object
clear classes
MIST('Crackle_Michelson.mist');
m = Michelson(6); 

%% Initial conditions

% Random uniform error in power [W]
p_error = 0.04;
% Imperfect mirror reflection coefficients
m.m1.R = 0.99; % arbitrary value
m.m2.R = 0.90; % arbitrary value
% Imperfect gains on AP and SP
gainap = 0.85; % arbitrary value
gainsp = 1.56; % arbitrary value

%% Scan through fringe pattern

clear AP_power SP_power
% Generate a linear set of data points
N = 500;
phi = linspace(-pi, pi, N);

% Move microscopic tuning of one arm, determine fringe pattern 
for i=1:N
    m.s3.phi = phi(i);  % set the space propagation phase
    % compute the probe at AP, add uniform noise
    AP_power(i) = gainap*uniform_noise(p_error,m.asy());   
    % compute the probe at SP, add uniform noise
    SP_power(i) = gainsp*uniform_noise(p_error,m.sy());   
end

% Plot Michelson fringes at S and AS ports
figure()
plot(phi,SP_power, phi, AP_power, phi, abs(SP_power-AP_power))
legend('SP', 'AP', 'ABS(SP-AP)')
xlabel('Phase of movable arm [rad]', 'FontSize', 18)
ylabel('Power [W]', 'FontSize', 18)
grid on
hold all

%% Adjust relative gain

% Measure mean power, then adjust power gain of port with smaller gain
gaincompensate = real(mean(SP_power)./mean(AP_power)); %SP/AP ratio

if gaincompensate>1 %SP>AP
    gainap = gaincompensate; % boost AP power
    AP_power = gainap*AP_power;
else %SP<AP
    gainsp = 1./gaincompensate; % boost SP power
    SP_power = gainsp*SP_power;
end

% Plot Michelson fringes at S and AS ports
figure()
plot(phi,SP_power, phi, AP_power, phi, abs(SP_power-AP_power))
legend('SP', 'AP', 'ABS(SP-AP)')
xlabel('Phase of movable arm [rad]', 'FontSize', 18)
ylabel('Power [W]', 'FontSize', 18)
grid on
hold all

%% Locking mechanism

% Lock system at half fringe condition
dofname = 'd2'; % define object that is to be locked, in this case mirrors
errsig = @() gainap*m.asy() - gainsp*m.sy(); % error signal, with error in 
% port gain taken into account
gain = -0.02; % arbitrary value
actuators = {'s2.phi'}; % parameters to be corrected
driving = [-1]; % lock acting differentially on one arm
tolerance = 1e-12;

% Detune system by an arbitrary amount
m.s2.phi = 2e-7;
% Perform lock
[err,corr,act] = m.lock(dofname, errsig, gain, actuators, driving, tolerance, ...
    'plot', 'log', 'itermax', 100, 'gain', 1);